require 'rubygems'

if RUBY_ENGINE == 'jruby'
  require 'march_hare'
  RABBIT_MQ_ADAPTER = MarchHare
else
  require 'bunny'
  RABBIT_MQ_ADAPTER = Bunny
end

require_relative 'microbunny/settings'
require_relative 'microbunny/connection'
require_relative 'microbunny/record'
require_relative 'microbunny/subscriptions'
require_relative 'microbunny/event_lookup'
require_relative 'microbunny/event_trigger'
require_relative 'microbunny/event'
require_relative 'microbunny/payload'

module MicroBunny
  class << self
    attr_accessor :settings, :connection, :channel

    def configure
      self.settings ||= Settings.new
      yield(settings)
    end

    def start
      self.connection = Connection.new(
        adapter: RABBIT_MQ_ADAPTER,
        host: self.settings.host,
        vhost: self.settings.vhost,
        username: self.settings.username,
        password: self.settings.password
      )
      
      self.channel = self.connection.channel
    end
  
    def stop
      self.connection.stop
    end

    def subscribe(topics, record_ids = [])
      raise_bad_start unless self.channel

      records = record_ids.map(&Record.method(:new))
      subs = Subscriptions.new(self.channel, topics, records) do |payload|
        handle(payload) 
      end

      subs.create
    end

    def event(name)
      EventTrigger.new(channel, name)
    end

    private

    def handle(json)
      payload = Payload.deserialize(json) 
      consumer = EventLookup.new(payload.event).consumer.new

      if payload.record
        consumer.process(payload.message, payload.record)
      else
        consumer.process(payload.message)
      end
    end
   
    def raise_bad_start
      raise "Must call MicroBunny.start before you can subscribe" 
    end
  end
end

