require 'json'
require_relative './record'
require_relative './payload'

module MicroBunny
  class Event
    def self.exchange(name)
      define_method(:exchange) do
        name.to_sym
      end
    end

    def initialize(channel)
      @channel = channel
    end

    def publish(message, record_id = nil)
      x = channel.topic(self.exchange)
      payload = build_payload(name, message)

      if record_id
        record = Record.new(record_id)
        payload.record = record.id
        x.publish(payload.serialize, routing_key: record.routing_key_for(self))
      else
        x.publish(payload.serialize, routing_key: name)
      end
    end

    def name
      self.class.name.gsub(/Event$/, '')
    end

    private

    attr_reader :channel

    def build_payload(name, message)
      Payload.new(event: name, message: message)
    end

    def serialize(payload)
      payload.to_json
    end
  end
end



