module MicroBunny
  class Record
    attr_reader :label

    def initialize(record, settings = MicroBunny.settings)
      @record = record
      @label = extract_label(record)
      @settings = settings
    end

    def id
      if string?
        record
      elsif active_record?
        record.id
      else
        raise_invalid_record
      end
    end

    def routing_key_matcher
      "*.#{label}"
    end

    def routing_key_for(event)
      "#{event.name}.#{label}" 
    end

    def address
      "#{settings.consumer_app}-#{label}"
    end

    private

    attr_reader :record, :settings

    def extract_label(record)
      return record if string?
      return activerecord_label if active_record?
      raise_invalid_record
    end

    def string?
      record.is_a? String
    end

    def active_record?
      record.is_a? ActiveRecord::Base
    end

    def activerecord_label
      "#{record.class.name.downcase}-#{record.id}"
    end

    def raise_invalid_record
      raise "A record must be either a String or an ActiveRecord object"
    end
  end
end
