module MicroBunny
  class Subscriber
    def initialize(channel, exchange, callback)
      @channel = channel
      @exchange = exchange
      @callback = callback
      @settings = settings
    end

    def topic(topic)
      queue = find_queue(topic)
      queue.bind(exchange, routing_key: "*")
      queue.subscribe { |_info, _meta, payload| callback.call(payload) }
    end

    def records(records)
      records.each do |record|
        queue = find_queue_for_record(record)
        queue.bind(exchange, routing_key: record.routing_key_matcher)
        queue.subscribe { |_info, _meta, payload| callback.call(payload) }
      end
    end

    private

    attr_reader :channel, :exchange, :callback, :settings

    def find_queue(name)
      channel.queue(name)
    end

    def find_queue_for_record(record)
      find_queue(record.address)
    end
  end
end

